# -*- coding: utf-8 -*-

import unittest

import sys
sys.path.append('..')
from Page import *
from Rect import Rect
from Labels import *

class TestPage(unittest.TestCase):
    contentData = ((LABEL_CHAR, (1, 1, 6, (0, 0, 1, 1))),
                   (LABEL_CHAR, (2, 2, 7, (1, 0, 2, 1))),
                   (LABEL_CHAR, (3, 3, 8, (2, 0, 3, 1))),
                   (LABEL_CHAR, (4, 4, None, (3, 0, 4, 1))),
                   (LABEL_CHAR, (5, 5, None, (4, 0, 5, 1))),
                   (LABEL_TEXTLINE, (6, 9, [1], (0, 0, 1, 1))),
                   (LABEL_TEXTLINE, (7, 10, [2], (1, 0, 2, 1))),
                   (LABEL_TEXTLINE, (8, 10, [3], (2, 0, 3, 1))),
                   (LABEL_PARAGRAPH, (9, 11, [6], (0, 0, 1, 1))),
                   (LABEL_PARAGRAPH, (10, 11, [7, 8], (1, 0, 3, 1))),
                   (LABEL_BODY, (11, None, [9, 10], (0, 0, 3, 1)))
                   )
    relationData = ((6, [1]),
                    (7, [2]),
                    (8, [3]),
                    (9, [6]),
                    (10, [7, 8]),
                    (11, [9, 10])
                    )

    def setUp(self):
        self.page = Page()
        for label, params in self.contentData:
            if label in LABEL_LEAF:
                pid, lid, plid, boxParam = params
                self.page.create(label, pid=pid, lid=lid, plid=plid,
                                 box=Rect(*boxParam))
            elif label in LABEL_COMPOSITE:
                lid, plid, clids, boxParam = params
                self.page.create(label, lid=lid, plid=plid, clids=clids,
                                 box=Rect(*boxParam))
        for plid, clids in self.relationData:
            self.page.attach(plid, clids)

    def testCreate(self):
        self.page.create(LABEL_PATH, pid=6, lid=None, box=Rect(3, 3, 4, 4))
        self.assertEqual(self.page.count(), 12)

    def testCreateType(self):
        for index, leafLabel in enumerate(LABEL_LEAF):
            self.assertTrue(self.page.create(leafLabel, pid=100+index).isLeaf())
        for compositeLabel in LABEL_COMPOSITE:
            self.assertFalse(self.page.create(compositeLabel, clids=[]).isLeaf())
        self.assertRaises(LabelError, self.page.create, 'BadLabel', clids=[])

    def testCreateDuplicate(self):
        self.assertRaises(DupLidError, self.page.create, 'Char', lid=10)
        self.assertRaises(DupPidError, self.page.create, 'Char', pid=5)

    def testDelete(self):
        self.page.delete(4)
        self.assertEqual(self.page.count(), 10)

    def testDeleteBeforeDetach(self):
        self.assertRaises(DeleteBeforeDetachError, self.page.delete, 11)

    def testDeleteNotExist(self):
        self.assertRaises(DeleteNotExistError, self.page.delete, 0)

    def testGetByLid(self):
        contents = set()
        for lid in range(1, 12):
            contents.add(self.page.getByLid(lid))
        self.assertEqual(len(contents), 11)

    def testGetByLidNotExist(self):
        self.assertTrue(self.page.getByLid(-1) is None)
        self.assertTrue(self.page.getByLid(None) is None)

    def testGetByPid(self):
        contents = set()
        for pid in range(5):
            contents.add(self.page.getByPid(pid))
        self.assertEqual(len(contents), 5)

    def testGetByPidNotExist(self):
        self.assertTrue(self.page.getByPid(None) is None)

    def testAttach(self):
        self.page.attach(8, [4, 5])
        content = self.page.getByLid(4)
        self.assertEqual(content.getPlid(), 8)
        content = self.page.getByLid(5)
        self.assertEqual(content.getPlid(), 8)

    def testAttachNotExist(self):
        self.assertRaises(Exception, self.page.attach, 1, 100)
        self.assertRaises(Exception, self.page.attach, 100, 1)

    def testAttachBeforeDetach(self):
        self.assertRaises(AttachBeforeDetachError, self.page.attach, 9, [7])

    def testLidToPids(self):
        self.assertEqual(set(self.page.lidToPids(11)), set(range(1, 4)))

    def testLabelToPids(self):
        self.assertEqual(set(self.page.labelToPids(LABEL_TEXTLINE)),
                         set(range(1, 4)))

    def testLidToAncestors(self):
        self.assertEqual([c.getLid() for c in self.page.lidToAncestors(2)],
                         [7, 10, 11])

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()
